#!/usr/bin/python

# convert klaus schmidt automata format to faudes format

import sys, os, re


def comp_subdir_sig(subdir):
  return subdir

def traverse_dir(basedir):
  automatafiles = []
  subdirs = []
  dir_file_tuples = []
  filesexpr = re.compile('.*(]|_spec|_sup)\.txt$')
  for path, dirs, files in os.walk(basedir):
    subdir = path.replace(basedir, '')
    subdirsig = comp_subdir_sig(subdir)
    subdirs.append(subdir)
    for filename in files:
      if filesexpr.match(filename):
        file = path+"/"+filename
        automatafiles.append(file)
        dir_file_tuples.append((subdir,file, subdirsig))
  return subdirs, automatafiles, dir_file_tuples

def convert_automata_file(tuple, readdir, writedir):
  alphabet = []
  controllable = []
  forcible = []
  highlevel = []
  stateset = []
  transrel = []
  istates = []
  mstates = []
  ltransitions = []
  ltransitionsforcible = []

  subdir = tuple[0]
  filename = tuple[1]
  subdirsig = tuple[2]
  timed_event = "t_"+subdirsig

  print "process_automata_file(\""+filename+"\")"

  # add the timed event to each automaton
  alphabet.append(timed_event)
  controllable.append(timed_event);

  regexp_alph = re.compile('^([\w.+-]+)(?:\s+(\d))(?:\s+(\d)|)(?:\s+(\d)|)(?:\s+(\d)|)(?:\s+(\d)|)\s*$')
  f = open(filename)
  f.readline()
  line = f.readline()
  while 1:
    line = f.readline().rstrip()
    if not line:
      break
    if line == "~el":
      break
    print line
    m = regexp_alph.match(line)
    if m:
      if m.group(1):
        # alphabet
        print "event: "+m.group(1)
        alphabet.append(m.group(1))
      if m.group(2) == "1":
        # controllable
        print "controllable: "+m.group(2)
        controllable.append(m.group(1))
      if m.group(3):
        #print "group3: "+m.group(3)
        pass
      if m.group(4) == "1":
        # forcible
        print "forcible: "+m.group(4)
        forcible.append(m.group(1))
      if m.group(5) == "1":
        # high level event
        print "high level: "+m.group(5)
        highlevel.append(m.group(5))

  # stateset + transrel
  regexp_state = re.compile('^([\w.|-]+)\s+(\d)\s+(\d+)\s*$')
  regexp_trans = re.compile('^(\S+)\s+(\S+)\s*$')
  regexp_plant = re.compile('^.*\]\.txt')
  regexp_spec = re.compile('^.*_spec\.txt')
  regexp_sup = re.compile('^.*_sup\.txt')
  while 1:
    line = f.readline()
    if not line:
      break
    line = line.rstrip()
    if line == "&fsm":
      break
  states = int(f.readline().rstrip())
  print "states: "+str(states)
  for statenum in range(1, states+1):
    print
    f.readline()
    line = f.readline().rstrip()
    print "sline: "+line
    m = regexp_state.match(line)
    if m:
      if m.group(1):
        print "state: "+str(m.group(1))
        x1 = str(m.group(1))
        stateset.append(x1)
        # istates
        if (statenum == 1):
          istates.append(x1)
      if m.group(2):
        print "marked: "+str(m.group(2))
        # mstates
        if (int(m.group(2)) == 1):
          mstates.append(x1)
      if m.group(3):
        print "transitions: "+str(m.group(3))
        transitions = int(m.group(3))
    # transrel
    m = regexp_spec.match(filename)
    # if plant or supervisor
    if not m:
      ltransitions = []
      ltransitionsforcible = []
      for transitionnum in range(transitions):
        line = f.readline().rstrip()
        print "transition: "+line
        m = regexp_trans.match(line)
        if m:
          print "event: "+str(m.group(1))
          ev = str(m.group(1))
          print "nextstate: "+str(m.group(2))
          x2 = str(m.group(2))
          t = x1, ev, x2
          # forcible event
          if ev in forcible:
            ltransitionsforcible.append(t)
          # unforcible event
          else:
            ltransitions.append(t)

      # if there were no forcible transitions
      if len(ltransitionsforcible) == 0:
        for t in ltransitions:
          transrel.append("\""+t[0]+"\" \""+t[1]+"\" \""+t[2]+"\"")
      # if there were only forcible transitions
      elif len(ltransitions) == 0:
        for t in ltransitionsforcible:
          transrel.append("\""+t[0]+"\" \""+t[1]+"\" \""+t[2]+"\"")
      # else
      else:
        dummy_state = "dummy_"+t[0]
        stateset.append(dummy_state)
        transrel.append("\""+t[0]+"\" \""+timed_event+"\" \""+dummy_state+"\"")
        for t in ltransitions:
          transrel.append("\""+dummy_state+"\" \""+t[1]+"\" \""+t[2]+"\"")
        for t in ltransitionsforcible:
          transrel.append("\""+t[0]+"\" \""+t[1]+"\" \""+t[2]+"\"")

    # if specification
    else:
      transrel.append("\""+x1+"\" \""+timed_event+"\" \""+x1+"\"")
      for transitionnum in range(transitions):
        line = f.readline().rstrip()
        print "transition: "+line
        m = regexp_trans.match(line)
        if m:
          print "event: "+str(m.group(1))
          ev = str(m.group(1))
          print "nextstate: "+str(m.group(2))
          x2 = str(m.group(2))
          transrel.append("\""+x1+"\" \""+ev+"\" \""+x2+"\"")
  f.close()
  print "alphabet: "+str(alphabet)
  print "controllable events: "+str(controllable)
  print "forcable events: "+str(forcible)
  print "high level events: "+str(highlevel)
  print "stateset: "+str(stateset)
  print "transrel: "+str(transrel)
  print "istates: "+str(istates)
  print "mstates: "+str(mstates)

  newfilegen = writedir+filename.replace(readdir, '').replace(".txt", ".gen")
  newfilehighalph = writedir+filename.replace(readdir, '').replace(".txt", ".alp")
  regexp_genname = re.compile('^.*/(.+)\....$')
  m = regexp_genname.match(filename)
  if m:
    if m.group(1):
      print "group1: "+str(m.group(1))
      genname = str(m.group(1))
  print "newfilegen: "+newfilegen
  print "genname: "+genname


  # sort automaton members
  alphabet.sort()
  controllable.sort()
  highlevel.sort()
  stateset.sort()
  transrel.sort()
  istates.sort()
  mstates.sort()

#   # write high level alphabet
#   f = open(newfilehighalph, 'w')
#   f.write("<Alphabet>\n")
#   colcount = 0
#   for event in highlevel:
#     f.write("\""+event+"\" ")
#     colcount = colcount + 1
#     if colcount % 4 == 0:
#       f.write("\n")
#       colcount = 0
#   f.write("\n")
#   f.write("</Alphabet>\n")
#   f.close()

  # write high level alphabet
  f = open(newfilehighalph, 'w')
  f.write("<Alphabet>\n")
  colcount = 0
  for event in alphabet:
    f.write("\""+event+"\" ")
    colcount = colcount + 1
    if colcount % 4 == 0:
      f.write("\n")
      colcount = 0
  f.write("\n")
  f.write("</Alphabet>\n")
  f.close()
  
  
  #write generator
  f = open(newfilegen, 'w')
  f.write("<Generator>\n")
  f.write("\""+genname+"\"\n")
  f.write("\n")
  f.write("<Alphabet>\n")
  colcount = 0
  for event in alphabet:
    f.write("\""+event+"\" ")
    colcount = colcount + 1
    if colcount % 4 == 0:
      f.write("\n")
      colcount = 0
  f.write("\n")
  f.write("</Alphabet>\n")
  f.write("\n")
  f.write("<States>\n")
  colcount = 0
  for state in stateset:
    f.write("\""+state+"\" ")
    colcount = colcount + 1
    if colcount % 4 == 0:
      f.write("\n")
      colcount = 0
  f.write("\n")
  f.write("</States>\n")
  f.write("\n")
  f.write("<TransRel>\n")
  for transition in transrel:
    f.write(transition+"\n")
  f.write("\n")
  f.write("</TransRel>\n")
  f.write("\n")
  f.write("<InitStates>\n")
  colcount = 0
  for istate in istates:
    f.write("\""+istate+"\" ")
    colcount = colcount + 1
    if colcount % 4 == 0:
      f.write("\n")
      colcount = 0
  f.write("\n")
  f.write("</InitStates>\n")
  f.write("\n")
  f.write("<MarkedStates>\n")
  colcount = 0
  for mstate in mstates:
    f.write("\""+mstate+"\" ")
    colcount = colcount + 1
    if colcount % 4 == 0:
      f.write("\n")
      colcount = 0
  f.write("\n")
  f.write("</MarkedStates>\n")
  f.write("\n")
  f.write("</Generator>\n")
  f.write("\n")
  f.write("<EventProperties>\n")
  f.write("\n")
  f.write("<Controllable>\n")
  colcount = 1
  for cevent in controllable:
    f.write("\""+cevent+"\" ")
    if colcount % 4 == 0:
      f.write("\n")
      colcount = 0
    colcount = colcount + 1
  f.write("\n")
  f.write("</Controllable>\n")
  f.write("\n")
  f.write("</EventProperties>\n")
  f.close()
  

def create_new_dirs(basedir, subdirs):
  for subdir in subdirs:
    os.mkdir(basedir+subdir)






#######################################
# main
#######################################

#print "len(argv): "+str(len(sys.argv))

if len(sys.argv) != 3:
  print "usage: convert_automata.py <inputdir> <outputdir>"
  sys.exit(1)
else:
  readdir = sys.argv[1]
  writedir = sys.argv[2]

if not re.search('/$', readdir):
  readdir = readdir+"/"
if not re.search('/$', writedir):
  writedir = writedir+"/"
  
subdirs, automatafiles, dir_file_tuples = traverse_dir(readdir)
create_new_dirs(writedir, subdirs)
for tuple in dir_file_tuples:
  print "file: " + tuple[1]
  convert_automata_file(tuple, readdir, writedir)
