/* baseset.cpp -- base class for EventSet and StateSet */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

#include "baseset.h"

namespace faudes {

// Name
std::string BaseSet::Name(void) const {
	return mMyName;
}
		
// Name
void BaseSet::Name(const std::string& rName) {
	FD_DV("BaseSet::Name(" << rName.c_str() << ")");
	mMyName = rName;
}
  
// Size()
Idx BaseSet::Size(void) const {
	return (Idx)mIndexSet.size();
}

// Empty()
bool BaseSet::Empty(void) const {
	return mIndexSet.empty();
}

// Begin()
BaseSet::iterator BaseSet::Begin(void) {
	return mIndexSet.begin();
}

// Begin() const
BaseSet::const_iterator BaseSet::Begin(void) const {
	return mIndexSet.begin();
}
		
// End()
BaseSet::iterator BaseSet::End(void) {
	return mIndexSet.end();
}

// End()
BaseSet::const_iterator BaseSet::End(void) const {
	return mIndexSet.end();
}


// SetUnion(set)
void BaseSet::SetUnion(const BaseSet& otherset) {
	const_iterator it1 = Begin();
	const_iterator it2 = otherset.Begin();
	while ((it1 != End()) && (it2 != otherset.End())) {
		if (*it1 < *it2) {
			++it1;
		}
		else if (*it1 == *it2) {
			++it1;
			++it2;
		}
		else { // (*it1 > *it2)
			mIndexSet.insert(*it2);
			++it2;
		}
	}
	while (it2 != otherset.End()) {
		mIndexSet.insert(End(), *it2);
		++it2;
	}
}

// SetIntersection(set)
void BaseSet::SetIntersection(const BaseSet& otherset) {
	iterator it1 = Begin();
	const_iterator it2 = otherset.Begin();
	iterator tmpit;
	while ((it1 != End()) && (it2 != otherset.End())) {
		if (*it1 < *it2) {
			tmpit = it1;	
			++it1;
			mIndexSet.erase(tmpit);
		}
		else if (*it1 == *it2) {
			++it1;
			++it2;
		}
		else { // (*it1 > *it2)
			++it2;
		}
	}
	while (it1 != End()) {
		tmpit = it1;	
		++it1;
		mIndexSet.erase(tmpit);
	}
}


// operator ==
bool BaseSet::operator == (const BaseSet& otherset) const {
	// return equal(otherset);
	if (mIndexSet == otherset.mIndexSet) {
		return true;
	}
	else return false;
}

// operator != 
bool BaseSet::operator != (const BaseSet& otherset) const {
	if (mIndexSet != otherset.mIndexSet) {
		return true;
	}
	else return false;
}

// operator <=
bool BaseSet::operator <= (const BaseSet& otherset) const {
	if (std::includes(otherset.Begin(), otherset.End(), Begin(), End())) {
		return true;
	}
	else return false;
}

// operator >=
bool BaseSet::operator >= (const BaseSet& otherset) const {
	if (std::includes(Begin(), End(), otherset.Begin(), otherset.End())) {
		return true;
	}
	else return false;
}

// operator <
bool BaseSet::operator < (const BaseSet& otherset) const {
	if (mIndexSet < otherset.mIndexSet) {
		return true;
	}
	else return false;
}

// Signature()
Idx BaseSet::Signature(void) const {
  // prepare result
  Idx sig = 0;
  // helpers:
  Idx i = 1;
  const_iterator it;
  // algorithm:
  for (it = Begin(); it != End(); ++it) {
    sig += *it * i;
    ++i;
  }
  return sig;
}

} // namespace faudes


