/* cmap.cpp - state name composition for parallel compositions  */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "cmap.h"

namespace faudes {

// CompositionMap::CompositionMap()
CompositionMap::CompositionMap(void) {
	mpStateSymbolTable = StateSet::StaticStateSymbolTablep();
}

// Begin()
CompositionMap::iterator CompositionMap::Begin(void) {
	return mCompositionMap.begin();
}

// End()
CompositionMap::iterator CompositionMap::End(void) {
	return mCompositionMap.end();
}

// CompositionMap::InsReverseMap(rReverseCompositionMap)
void CompositionMap::InsReverseMap(
    const std::map< std::pair<Idx,Idx>, Idx >& rReverseCompositionMap) {
	std::map< std::pair<Idx,Idx>, Idx >::const_iterator it;
	for (it = rReverseCompositionMap.begin(); 
      it != rReverseCompositionMap.end(); ++it) {
		mCompositionMap[it->second] = it->first;
	}
}

// CompositionMap::StateName(index)
std::string CompositionMap::StateName(Idx index) const {
	std::map< Idx, std::pair<Idx,Idx> >::const_iterator it;
	std::string statename;
	it = mCompositionMap.find(index);
	if (it != mCompositionMap.end()) {
		return StateName(it->second.first) + "|" + StateName(it->second.second);
	}
	else {
		statename = mpStateSymbolTable->StateName(index);
		if (statename == "") {
			return ToString(index);
		}
		else
		{
			return statename;
		}
	}
}

// CompStateNames(Generator&)
void CompositionMap::CompStateNames(Generator& rGen) const {
	StateSet::iterator it;
	for (it = rGen.StatesBegin(); it != rGen.StatesEnd(); ++it) {
		rGen.StateName(*it, StateName(*it));
	}
}

// CompositionMap::DelState(index)
bool CompositionMap::DelState(Idx index) {
	std::map< Idx, std::pair<Idx,Idx> >::iterator it;
	it = mCompositionMap.find(index);
	if (it != mCompositionMap.end()) {
		DelState(it->second.first);
		DelState(it->second.second);
		mCompositionMap.erase(index);
		return true;
	}
	else {
		return false;
	}
}

// CompositionMap::DelStates(rGen)
bool CompositionMap::DelStates(const Generator& rGen) {
	StateSet::const_iterator it;
	for (it = rGen.StatesBegin(); it != rGen.StatesEnd(); ++it) {
		if (! DelState(*it)) {
			return false;
		}
	}
	return true;
}
		
} // namespace faudes
