/* cparallel.cpp -- parallel composition */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "cparallel.h"

/* this Macro for cast from derived class of Generator to Generator */ 
#define CAST_GEN *static_cast<Generator*>
#define CONSTCAST_GEN *static_cast<const Generator*>

namespace faudes {

// Parallel(rGen1, rGen2, res)
void Parallel(const cGenerator& rGen1, const cGenerator& rGen2, 
    cGenerator& rResGen){
    FD_DF("cParallel(" << &rGen1 << "," << &rGen2 << ")");
    rResGen.StateFlagTabler().Clear();
    aFlagTable EventFlagTable;
    cComposedEventFlagTable(rGen1,rGen2,EventFlagTable);
    Parallel(CONSTCAST_GEN(&rGen1),CONSTCAST_GEN(&rGen2),CAST_GEN(&rResGen) );
    rResGen.SetEventFlagTable(EventFlagTable);
}

// Parallel(rGen1, rGen2, rReverseCompositionMap, res)
void Parallel(const cGenerator& rGen1, const cGenerator& rGen2,
    std::map< std::pair<Idx,Idx>, Idx>& rReverseCompositionMap, 
    cGenerator& rResGen){
    FD_DF("cParallel(" << &rGen1 << "," << &rGen2 << "," << &rReverseCompositionMap << ")");
    rResGen.StateFlagTabler().Clear();
    aFlagTable EventFlagTable;
    cComposedEventFlagTable(rGen1,rGen2,EventFlagTable);
    Parallel(CONSTCAST_GEN(&rGen1),CONSTCAST_GEN(&rGen2),rReverseCompositionMap,CAST_GEN(&rResGen) );
    rResGen.SetEventFlagTable(EventFlagTable);    
} 
    
// cComposedEventFlagTable(rGen1,rGen2)
void cComposedEventFlagTable(const cGenerator& rGen1, const cGenerator& rGen2, aFlagTable& rEventFlagTable){
    FD_DF("cComposedEventFlagTable(" << &rGen1 << "," << &rGen2 << ")");
    // HELPERS:
    std::map<Idx,aType>::const_iterator rGen1FlagIterator;
    std::map<Idx,aType>::const_iterator rGen2FlagIterator;
    rGen1FlagIterator = rGen1.EventFlagTableBegin();
    rGen2FlagIterator = rGen2.EventFlagTableBegin();
    
    while(1){
        FD_DF("cParallel: investigated events: " << 
        EventSet::StaticEventSymbolTablep()->EventName(rGen1FlagIterator->first) << " and " 
        << EventSet::StaticEventSymbolTablep()->EventName(rGen2FlagIterator->first) 
        << std::endl);

        if( rGen1FlagIterator == rGen1.EventFlagTableEnd() &&
                rGen2FlagIterator == rGen2.EventFlagTableEnd() ){
                break;
        }
        else if ( rGen1FlagIterator == rGen1.EventFlagTableEnd() &&
                rGen2FlagIterator != rGen2.EventFlagTableEnd() ){
            rEventFlagTable.SetFlags(rGen2FlagIterator->first,rGen2FlagIterator->second);
            rGen2FlagIterator++;                
        }  
        else if ( rGen1FlagIterator != rGen1.EventFlagTableEnd() &&
                rGen2FlagIterator == rGen2.EventFlagTableEnd() ){
            rEventFlagTable.SetFlags(rGen1FlagIterator->first,rGen1FlagIterator->second);
            rGen1FlagIterator++;                
        }
        else if( rGen1FlagIterator->first < rGen2FlagIterator->first ){
            rEventFlagTable.SetFlags(rGen1FlagIterator->first,rGen1FlagIterator->second);
            rGen1FlagIterator++;
        }
        else if( rGen1FlagIterator->first == rGen2FlagIterator->first ){
            if(rGen1FlagIterator->second == rGen2FlagIterator->second){   
                rEventFlagTable.SetFlags(rGen1FlagIterator->first,rGen1FlagIterator->second);
                rGen1FlagIterator++;
                rGen2FlagIterator++;
            }
            else{
     			std::stringstream errstr;
     			errstr << "Mismatch of event properties";
				throw Exception("composedEventFlagTable", errstr.str(), 100);
            }
        }
        else{             
            rEventFlagTable.SetFlags(rGen2FlagIterator->first,rGen2FlagIterator->second);
            rGen2FlagIterator++;
        }
    }  
}


//InvProject(rGen1, rGen2)
void InvProject(cGenerator& rGen1, const cGenerator& rGen2){
    FD_DF("InvProject(" << &rGen1 << "," << &rGen2 << ")");
    
    try{
        InvProject(rGen1,rGen2.Alphabet(),rGen2.ControllableEvents(),rGen2.ObservableEvents() );
    }
    catch(Exception& ex){
        std::stringstream errstr;
        errstr << ex.What();
        throw Exception("InvProject(cGenerator,cGenerator)", errstr.str(), 506);
    }  
}

// InvProject(rGen1,rAlphabet,rControllable,rObservable)
void InvProject(cGenerator& rGen1, const EventSet& rAlphabet, 
    const EventSet rControllable, const EventSet rObservable ){
    FD_DF("InvProject(" << &rGen1 << "," << &rAlphabet << "," << &rControllable << "," << &rObservable << ")");
    aFlagTable eventTable = rGen1.EventFlagTable();
    aFlagTable stateTable = rGen1.StateFlagTable();
    try{
        InvProject(CAST_GEN(&rGen1),rAlphabet);
        rGen1.SetStateFlagTable(stateTable);
        rGen1.SetEventFlagTable(eventTable);
    }
    catch(Exception& ex){
        std::stringstream errstr;
        errstr << ex.What();
        throw Exception("InvProject(cGenerator,EventSet)", errstr.str(), 506);
    }    
    EventSet::const_iterator iterator;
    EventSet alphabetDifference = rAlphabet - rGen1.Alphabet();
    EventSet Uncontrollable = rAlphabet - rControllable;
    EventSet Unobservable = rAlphabet - rObservable;
    for(iterator = rControllable.Begin(); iterator != rControllable.End();iterator++){
        rGen1.SetControllable(*iterator);
    }
    for(iterator = rObservable.Begin(); iterator != rObservable.End();iterator++){
        rGen1.SetObservable(*iterator);
    }
    for(iterator = Uncontrollable.Begin(); iterator != Uncontrollable.End();iterator++){
        rGen1.SetUncontrollable(*iterator);
    }
    for(iterator = Unobservable.Begin(); iterator != Unobservable.End();iterator++){
        rGen1.SetUnobservable(*iterator);
    }
}

} // namespace faudes
