/* debug.hpp -- macros for debugging */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

#ifndef DEBUG_H

#include <iostream>

// fallback version string
#ifndef FAUDES_VERSION
#define FAUDES_VERSION "undefined version"
#endif

// always print warnings
#define FD_WARN(message) std::cerr << "FAUDES_WARNING: " << message << std::endl
//#define FAUDES_CHECKED
// print errors with file and line
#ifdef FAUDES_DEBUG_CODE
#define FD_ERR(message) std::cerr << "FAUDES_ERROR: " << message << " in " \
		<< __FILE__ << ":" << __LINE__ << std::endl
#else
#define FD_ERR(message)
#endif

// debugging: write process to console
#ifdef FAUDES_WRITE_PROGRESS // PR = Progress
#define FD_WP(message) std::cout << message << std::endl
#else
#define FD_WP(message)
#endif

// debugging: smartpointer debug level
#ifdef FAUDES_DEBUG_MEMORY // M = Memory
#define FD_DM(message) std::cerr << "FAUDES_MEMORY: " << message << std::endl
#else
#define FD_DM(message)
#endif

// debugging: function level
#ifdef FAUDES_DEBUG_FUNCTION
#define FD_DF(message) std::cerr << "FAUDES_FUNCTION: " << message << std::endl
#else
#define FD_DF(message)
#endif


// debugging: generator level
#ifdef FAUDES_DEBUG_GENERATOR
#define FD_DG(message) std::cerr << "FAUDES_GENERATOR: " << message << std::endl
#else
#define FD_DG(message)
#endif

// debugging: container level: eset, lset, transrel
#ifdef FAUDES_DEBUG_CONTAINER
#define FD_DC(message) std::cerr << "FAUDES_CONTAINER: " << message << std::endl;
#else
#define FD_DC(message)
#endif

// debugging: verbose level: lower iterations, token i/o
#ifdef FAUDES_DEBUG_VERBOSE
#define FD_DV(message) std::cerr << "FAUDES_VERBOSE: " <<  message << std::endl;
#else
#define FD_DV(message)
#endif

#define DEBUG_H
#endif 
