/* statemin.h -- state space minimization */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef STATEMIN_H

#include "definitions.h"
#include "debug.h"
#include "generator.h"
#include <vector>
#include <string>
#include <sstream>

namespace faudes {

/**
 * State Minimization
 * This function implements the (n*log n) set partitioning algorithm by
 * John E. Hopcroft. Given generator will be made accessible before computing
 * minimized generator.
 *
 * @param rGen
 *   Generator
 * @param rResGen
 *   Minimized generator (result)
 *
 * @exception Exception
 *   Input automaton nondeterministic (id 505)
 */
void StateMin(Generator& rGen, Generator& rResGen);

/**
 * State Minimization
 * This function implements the (n*log n) set partitioning algorithm by
 * John E. Hopcroft. Given generator will be made accessible before computing
 * minimized generator.
 *
 * @param rGen
 *   Generator
 * @param rResGen
 *   Minimized generator (result)
 * @param rSubsets
 *   Vector of subsets that will be constructed during running the algorithm
 *   (optional parameter)
 * @param rNewIndices
 *   Vector of new state indices corresponding to the subsets
 *   (optional parameter)
 *
 * @exception Exception
 *   Input automaton nondeterministic (id 505)
 */
void StateMin(Generator& rGen, Generator& rResGen, 
		std::vector<StateSet>& rSubsets, std::vector<Idx>& rNewIndices);

} // namespace faudes

#define STATEMIN_H
#endif 
