/* stringhelper.cpp -- helper functions for strings */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#include "stringhelper.h"

namespace faudes {

// ToString(index)
std::string ToString(Idx index) { 
   std::string res;
   std::stringstream sstr;
   sstr << index;
   sstr >> res;
   return res;
}

// ExpandString(rString, len) 
std::string ExpandString(const std::string& rString, unsigned int len) {
  std::string res;
  res	= rString;
  std::string::size_type xtra = (std::string::size_type) len - rString.length();
  if ((xtra > 0) && (xtra < 10000)) {
	  res.append(xtra, ' ');
	}
  return res;
} 

// ToIdx(rString)
Idx ToIdx(const std::string& rString) {
	char * end;
	unsigned long ul = strtoul (rString.c_str(), &end, 0);
	unsigned long idxmax = std::numeric_limits<Idx>::max();
	if (ul > idxmax) {
		throw Exception("atoidx", "Idx overflow", 600);
	}
	else {
		return (Idx) ul;
	}
}

// FDVersionToString()
std::string FDVersionToString() {
	return std::string(FAUDES_VERSION);
}

} // namespace faudes
