/* stringhelper.h -- helper functions for strings */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2006  Bernd Opitz
   Exclusive copyright is granted to Klaus Schmidt

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


#ifndef STRINGHELPER_H

#include "definitions.h"
#include "debug.h"
#include "exception.h"
#include <string>
#include <sstream>
#include <limits>

namespace faudes {

/**
 * Convert Idx to string
 * 
 * @param index
 *   Idx 
 *
 * @return
 *   string
 */
std::string ToString(Idx index);


/**
 * Fill string with spaces up to a given length if length of the string
 * is smaller than given length parameter.
 *
 * @param rString
 *   string
 * @param len
 *   Minimum number of charakters in string
 *
 * @return
 *   Expanded string
 */
std::string ExpandString(const std::string& rString, unsigned int len);


/**
 * Convert a string to Idx
 *
 * @param rString
 *   string to convert
 *
 * @return 
 *   Idx
 *
 * @exception
 *   Idx overflow (id 600)
 */
Idx ToIdx(const std::string& rString);

/**
 * Return FAUDES_VERSION als std::string
 *
 * @return 
 *   std::string with FAUDES_VERSION
 */
std::string FDVersionToString();

} // namespace faudes

#define STRINGHELPER_H
#endif

