//////////////////////////////////////////////////////////////////////////////
// 4_cgenerator_example.cpp --
//   Walkthrough of the cGenerator methods.
//   It is assumed the user is already familiar with the Generator class.
//////////////////////////////////////////////////////////////////////////////


#include "libfaudes.hpp"


// for simplicity we make the faudes namespace available to our program
using namespace faudes;


/*

General Description of the cGenerator Class:

The cGenerator class is inherited from the aGenerator class and only
differs by definition of the controllabilty flag for events. The
aGenerator class is a general purpose Generator class to assign attributs
to states and events.

Because of the inheritance, the cGenerator class's methods are a superset
of the Generator and aGenerator methods. Some of the methods, especially
for file i/o  are declared virtual and have a different implementation in
the cGenerator class.

The controllabilty property of events is hold by the mEventFlagTable
member that is inherited from the aGenerator. It is set and changed by
cGenerator methods. Changing the controllabilty property of an event in one
cGenerator does not change the event property of the same event in a different
cGenerator. Therefore algorithms that deal with controllabilty have to do a
consistency check each time the algorithm is executed.

If both classes, Generator and cGenerator are used there may be cases that
an event is not flagged controllable or uncontrollable at all. For example
by creating a Generator or EventSet with a new event. In such cases the
unflagged events are assumed to be controllable by default. There exists a
cGenerator method for checking if an event has an entry in the eventflagtable.

When reading cGenerator from files, events in the file that are already
known in the event domain may have a different control property in the file
and in the existing entry in the eventflagtable. To prevent further errors,
an exception may be thrown in such a case. Whether such exceptions are
thrown is configurable by a static cGenerator method.

*/



/////////////////
// main program
/////////////////

int main() {


  ////////////////////////////////////////////
  // Constructors & Assignment Operator
  ////////////////////////////////////////////

  // Create an empty cGenerator object

  cGenerator cgen1;

  // We can also create a cGenerator by reading a Generator file that
  // has a section of controllable events

  cGenerator cgen2("data/csimplemachine.gen");

  // Copy constructor & assignment operator
  // (same behaviour as in Generator class)

  cGenerator cgen3(cgen2);
  cgen3 = cgen2; // new indices



  ////////////////////////
  // Controllable events
  ////////////////////////

  // Insert an event and set it controllable (may already be flagged
  // uncontrollable)

  cgen1.InsControllableEvent(1); // by index
  cgen1.InsControllableEvent("newcevent1"); // by symbolic name

  // Insert an event and set it uncontrollable (may already be flagged
  // controllable)

  cgen1.InsUncontrollableEvent(2); // by index
  cgen1.InsUncontrollableEvent("newcevent2"); // by symbolic name

  // Set an existing event controllable

  cgen1.SetControllable(2); // by index
  cgen1.SetControllable("newcevent2"); // by symbolic name

  // Set an existing event uncontrollable

  cgen1.SetUncontrollable(1); // by index
  cgen1.SetUncontrollable("newcevent1"); // by symbolic name

  // Test wether an event is controllable

  bool bool_eventcontrollable = cgen1.Controllable(1); // by index
  bool_eventcontrollable = cgen1.Controllable("newcevent1"); // by symbolic name

  // Retrieve an EventSet containing all the controllabe events of the
  // cGenerator

  EventSet eset_cevents = cgen2.ControllableEvents();

  // Retrieve an EventSet containing all the uncontrollabe events of the
  // cGenerator

  EventSet eset_ucevents = cgen2.UncontrollableEvents();

  return 0;
}



