//////////////////////////////////////////////////////////////////////////////
// 6_agenerator_example.cpp --
//   Walkthrough of the aGenerator methods.
//   It is assumed the user is already familiar with the Generator class.
//////////////////////////////////////////////////////////////////////////////


#include "libfaudes.hpp"


// for simplicity we make the faudes namespace available to our program
using namespace faudes;


/*

General Description of the aGenerator Class:

The aGenerator class is inherited from the Generator class and provides
boolean attribute tables for events and states.

Because of the inheritance, the aGenerator class's methods are a superset
of the Generator methods. Some of the methods, especially
for file i/o  are declared virtual and have a different implementation in
the aGenerator class.

The state and event attributes are held by the mEventFlagTable and the
mStateFlagTable, respectively. They can be changed and set using aGenerator
methods. As properties are only valid locally, changing an event
attribute in one aGenerator does not change the attribute for the same event
in another aGenerator.

*/



/////////////////
// main program
/////////////////

int main() {


	///////////////////////////////////////
	// Constructors & Assignment Operator
	///////////////////////////////////////

	// Create an empty aGenerator object

	aGenerator agen1;

	// We can also create an aGenerator by reading a Generator file that
	// has a section with Event Attributes and a section with State Attributes

	aGenerator agen2("asimplemachine.gen");

	// Copy constructor & assignment operator
	// (same behaviour as in Generator class)

	aGenerator agen3(agen2);
	agen3 = agen2; // new indices



	///////////////////////////////
	// Event and State Attributes
	///////////////////////////////

	// Insert an event and give it an attribute

	aType event_attribute = 0x00110001;
	agen1.InsAttributedEvent(1,event_attribute); // by index
	agen1.InsAttributedEvent("newaevent1",event_attribute); // by symbolic name

	// Insert a state and give it an attribute

	aType state_attribute = 0x00110011;
	agen1.InsAttributedState(2,state_attribute); // by index
	agen1.InsAttributedState("newastate1",state_attribute); // by symbolic name

	// Set the attributed of an existing event

	event_attribute = 0x01010101;
	agen1.SetEventAttribute(1,event_attribute); // by index
	agen1.SetEventAttribute("newaevent1",event_attribute); // by symbolic name

	// Set an existing event uncontrollable

	state_attribute = 0x00001111;
	agen1.SetStateAttribute(1,state_attribute); // by index

	// Test wether an event attribute is set

	aType check_attribute = 0x00000001;
	bool bool_eventattribute = (agen1.GetEventAttribute(1,check_attribute,0x00000000) == check_attribute); // by index
	bool_eventattribute = (agen1.GetEventAttribute("newaevent1",check_attribute,0x00000000) == check_attribute); // by symbolic name

	return 0;
}



